#!/usr/bin/env python2
# -*- coding=utf-8 -*-

try:
	from PyKDE4.kdecore import *
	from PyKDE4.kdeui import *
except ImportError:
	print """
	Kelogviewer requires pykde4 and portage 2.1.
	Vous n'avez pas installe pykde4.

	emerge pykde4
	"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
import sys, os, portage
import time, re


try:
	os.chdir(portage.settings["PORT_LOGDIR"])
except OSError:
	os.chdir("/var/log/portage")
try:
	os.chdir("./elog")
except OSError:
	print """
	Kelogviewer requires portage 2.1.

	It seems you do not have the elog facility on your system:
If you are already using portage 2.1, you need to edit your /etc/make.conf and add at least:

	PORTAGE_ELOG_CLASSES="info warn error log"
	PORTAGE_ELOG_SYSTEM="save"

Please refer to /etc/make.conf.example for more info.

	Il semble que vous n'avez pas d'elogs sur votre systeme. elogs est une nouvelle fonction \
de portage 2.1. Si vous utilisez deja portage 2.1, vous avez besoin d'editer votre fichier /etc/make.conf \
et d'y ajouter au moins les lignes suivantes:

	PORTAGE_ELOG_CLASSES="info warn error log"
	PORTAGE_ELOG_SYSTEM="save"

Voir /etc/make.conf.example pour plus d'options.
"""

class Logentry:

	def __init__(self, filename):
		intimeformat = "%Y%m%d-%H%M%S.log"
		outtimeformat = "%Y-%m-%d %H:%M:%S"

		(c, p, t) = re.split(":", str(filename))
		self.category = c
		self.package = p
		t = time.strptime(t, intimeformat)
		t = time.strftime(outtimeformat, t)
		self.timestamp = t
		self.filename = filename

class KelogInfo(QTextEdit):

	def __init__(self, parent):
		QTextEdit.__init__(self, parent)
		self.setReadOnly(True)
		self.setUndoRedoEnabled(False)

	def strip_nl(self, line):
		return re.match(r'^([^\\\n]*)', line).group(1)

	def parselog(self, filename, efilter):

		first = True
		headerpattern = re.compile('^(INFO|WARN|ERROR|LOG): [a-z]')
		self.clear()
		try:
			logfile = open(str(filename), 'r')
			for readline in logfile:
				readline = self.strip_nl(readline)
				if headerpattern.match(readline):
					l = readline.split(": ")
					header = l[0]
					stage = l[1]
					self.setTextColor(efilter.color(header, stage))
					if not first:
						readline = "\n" + readline
					else:
						first = False

				if efilter.filter(header) and efilter.filter(stage):
					if readline is not "":
						self.append(readline)
		except IOError:
			print "No file to read"

		self.setAlignment(Qt.AlignLeft)

class KelogFilter(QWidget):

	filter_headers_labels = {"INFO": QColor(0, 0, 0), "WARN": QColor(0, 0, 255),
			"ERROR": QColor(255, 0, 0), "LOG": QColor(0, 155, 0)}
	filter_stage_labels = ["preinst", "unpack", "setup", "compile", "install", "postinst",
			"test", "other", "prerm", "postrm"]

	def __init__(self, parent):
		QWidget.__init__(self, parent)
		self.layout = QVBoxLayout(self)
		self.headersLay = QGridLayout()
		self.stagesLay = QGridLayout()
		self.groupHeaders = QGroupBox(self)
		self.groupStage = QGroupBox(self)
		self.butHeaders = QButtonGroup(self)
		self.butHeaders.setExclusive(False)
		self.butStage = QButtonGroup(self)
		self.butStage.setExclusive(False)
		self.spacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
		self.layout.addWidget(self.groupHeaders)
		self.layout.addWidget(self.groupStage)
		self.layout.addItem(self.spacer)
		self.cb = {}
		for no, label in enumerate(sorted(self.filter_headers_labels.keys())):
			row = no / 2
			col = no % 2
			self.cb[label] = QCheckBox(label)
			self.headersLay.addWidget(self.cb[label], row, col)
			self.butHeaders.addButton(self.cb[label])
		for no, label in enumerate(sorted(self.filter_stage_labels)):
			row = no / 2
			col = no % 2
			self.cb[label] = QCheckBox(label)
			self.stagesLay.addWidget(self.cb[label], row, col)
			self.butStage.addButton(self.cb[label])
		self.groupHeaders.setLayout(self.headersLay)
		self.groupStage.setLayout(self.stagesLay)
		self.checkAll()
		self.connect(self.butHeaders, SIGNAL("buttonClicked(int)"), self.slotButtonClicked)
		self.connect(self.butStage, SIGNAL("buttonClicked(int)"), self.slotButtonClicked)

	def checkAll(self):
		for label in self.filter_headers_labels.keys():
			self.cb[label].setChecked(True)
		for label in self.filter_stage_labels:
			self.cb[label].setChecked(True)
		self.emit(SIGNAL("filterChanged"), ())

	def toggle(self):
		for label in self.filter_headers_labels.keys():
			self.cb[label].setChecked(not self.cb[label].isChecked())
		for label in self.filter_stage_labels:
			self.cb[label].setChecked(not self.cb[label].isChecked())
		self.emit(SIGNAL("filterChanged"), ())

	def filter(self, lbl):
		tmp = self.cb.get(lbl)
		if tmp:
			return tmp.isChecked()
		else:
			return True

	def color(self, header, stage):
		return self.filter_headers_labels[header]

	def slotButtonClicked(self, index):
		self.emit(SIGNAL("filterChanged"), ())

class KelogList(QTreeWidget):

	NO_MATCH = 0
	NO_OTHER_MATCH = -1
	MATCH = 1

	_tree = {}
	_tree_view = True

	def __init__(self, parent, viewmode):
		QTreeWidget.__init__(self, parent)

		self.setTreeViewMode(viewmode)

		self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
		self.setSortingEnabled(True)
		self.setAllColumnsShowFocus(True)
		self.setSelectionMode(QAbstractItemView.SingleSelection)
		self.setHeaderLabels(["Category", "Package", "Timestamp"])

		self.populate()
		for i in xrange(self.columnCount()):
			self.resizeColumnToContents(i)

	def setTreeViewMode(self, b):
		self._tree_view = b
		self.setRootIsDecorated(self._tree_view)

	def expand(self):
		it = QTreeWidgetItemIterator(self)
		itm = it.value()
		while itm:
			itm.setExpanded(True)
			it += 1
			itm = it.value()

	def collapse(self):
		it = QTreeWidgetItemIterator(self)
		itm = it.value()
		while itm:
			itm.setExpanded(False)
			it += 1
			itm = it.value()

	def refresh(self):
		self.clear()
		if (self._tree_view):
			self._tree = {}
		self.populate()

	def populate(self):
		for f in os.listdir(os.getcwd()):
			try:
				log = Logentry(f)
				if (self._tree_view):
					if (not self._tree.has_key(log.category)):
						#print "No key found, creating : ", log.category
						self._tree[log.category] = QTreeWidgetItem(self, [log.category, "", "", ""])
					QTreeWidgetItem(self._tree[log.category], [log.category, log.package, log.timestamp, log.filename])
				else:
					QTreeWidgetItem(self, [log.category, log.package, log.timestamp, log.filename])
			except:
				print "Exception occured while loading: ", f
		self.sortItems(2, Qt.DescendingOrder)
		self.setCurrentItem(self.topLevelItem(0))

	def deleteElog(self, item):
		if not item:
			return False
		if item.text(3) == "":
			return False
		self.takeItem(item)
		#print "Delete ", item.text(3)
		os.remove(str(item.text(3)))
		self.setSelected(self.currentItem(), True)
		return True

	def deleteAllElogs(self):
		it = QTreeWidgetItemIterator(self)
		itm = it.value()
		while itm:
			if itm.text(3):
				#print "Delete ", itm.text(3)
				os.remove(str(itm.text(3)))
			it += 1
			itm = it.value()
		self.refresh()

	def search(self, text):
		current = self.currentItem()
		res = self.findItems(text,  Qt.MatchContains, 1)
		# bail out early if there's no results
		if not res:
			return self.NO_MATCH

		# if it's the list, we're probably searching further
		if current in res:
			cur_index = res.index(current)
			# get the next result
			try:
				found = res[cur_index + 1]
				self.setCurrentItem(found)
				self.scrollToItem(found)
				return self.MATCH
			except IndexError:
				return self.NO_OTHER_MATCH
		else:
			found = res[0]
			self.setCurrentItem(found)
			self.scrollToItem(found)
			return self.MATCH


class Kelogviewer(KXmlGuiWindow):

	VIEW_MODE_LIST = 0
	VIEW_MODE_TREE = 1

	_view_mode = VIEW_MODE_LIST

	def __init__(self, *args):
		KXmlGuiWindow.__init__(self, *args)

		self.setGeometry(0, 0, 600, 600)
		self.setWindowIcon(KIcon("utilities-log-viewer"))

		# define actions
		self.initActions()
		# build the menu
		self.initMenus()
		# init the tool bar
		self.initToolBar()
		# init the widgets
		self.initWidgets()
		# statusBar
		self.initStatusBar()

		# Set Signals
		self.connect(self.elogList, SIGNAL("itemSelectionChanged()"), self.slotElogSelected)
		self.connect(self.elogFilter, SIGNAL("filterChanged"), self.slotElogFilterChanged)
		self.connect(self.elogSearch, SIGNAL("returnPressed()"), self.slotSearch)
		self.connect(self.btSearch, SIGNAL("clicked()"), self.slotSearch)

		self.slotElogSelected()

	def initActions (self):
		# "File" menu items
		self.refreshAction = KAction (KIcon("view-refresh"), i18n("&Refresh"), self.actionCollection())
		self.refreshAction.setShortcut(KShortcut(Qt.Key_F5))
		QObject.connect(self.refreshAction, SIGNAL("triggered()"), self.slotRefresh)
		self.deleteAction = KAction (KIcon("edit-delete"), i18n("&Delete"), self.actionCollection())
		QObject.connect(self.deleteAction, SIGNAL("triggered()"), self.slotDelete)
		self.deleteAction.setShortcut(KShortcut(Qt.Key_Delete))
		self.clearAllAction = KAction (KIcon("edit-clear-list"), i18n("&Clear All"), self.actionCollection())
		QObject.connect(self.clearAllAction, SIGNAL("triggered()"), self.slotClearAll)
		self.quitAction = KStandardAction.quit (self.slotQuit, self.actionCollection())

		# "Filter" menu items
		self.checkAllAction = KAction (i18n("Check &All"), self.actionCollection())
		QObject.connect(self.checkAllAction, SIGNAL("triggered()"), self.slotCheckAll)
		self.toggleAction = KAction (i18n("&Toggle"), self.actionCollection())
		QObject.connect(self.toggleAction, SIGNAL("triggered()"), self.slotToggle)

		# "View" menu items
		self.expandTreeAction = KAction (i18n("&Expand Tree View"), self.actionCollection())
		QObject.connect(self.expandTreeAction, SIGNAL("triggered()"), self.slotExpandTree)

		self.collapseTreeAction = KAction (i18n("&Collapse Tree View"), self.actionCollection())
		QObject.connect(self.collapseTreeAction, SIGNAL("triggered()"), self.slotCollapseTree)

		self.switchViewModeAction = KAction (i18n("&Switch View Mode"), self.actionCollection())
		QObject.connect(self.switchViewModeAction, SIGNAL("triggered()"), self.slotSwitchViewMode)

		self.deleteAction.setEnabled(False)
		self.clearAllAction.setEnabled(False)

		if (self._view_mode == self.VIEW_MODE_TREE):
			self.collapseTreeAction.setEnabled(True)
			self.expandTreeAction.setEnabled(True)
		else:
			self.collapseTreeAction.setEnabled(False)
			self.expandTreeAction.setEnabled(False)

	def initMenus (self):
		fileMenu = QMenu (i18n ("&File"), self)
		fileMenu.addAction(self.refreshAction)
		fileMenu.addAction(self.deleteAction)
		fileMenu.addAction(self.clearAllAction)
		fileMenu.addSeparator()
		fileMenu.addAction(self.quitAction)
		self.menuBar().addMenu(fileMenu)

		filterMenu = QMenu(i18n("&Filter"), self)
		filterMenu.addAction(self.checkAllAction)
		filterMenu.addAction(self.toggleAction)
		self.menuBar().addMenu (filterMenu)

		viewMenu = QMenu(i18n("&View"), self)
		viewMenu.addAction(self.expandTreeAction)
		viewMenu.addAction(self.collapseTreeAction)
		viewMenu.addSeparator()
		viewMenu.addAction(self.switchViewModeAction)
		self.menuBar().addMenu(viewMenu)

		helpMenu = KHelpMenu(self, KCmdLineArgs.aboutData(), True)
		self.menuBar().addMenu(helpMenu.menu())

	def initToolBar(self):
		self.toolBar().addAction(self.refreshAction)
		self.toolBar().addAction(self.deleteAction)
		self.toolBar().addAction(self.clearAllAction)
		self.toolBar().addAction(self.quitAction)

	def initWidgets(self):
		# define the layout
		self.mainBox = QSplitter(Qt.Vertical, self)
		self.setCentralWidget(self.mainBox)
		self.topBox = QVBoxLayout(self.mainBox)

		# Search widget
		self.searchBar = QWidget(self.mainBox)
		self.searchLay = QHBoxLayout(self.searchBar)
		self.searchLay.addWidget(QLabel("Search"))
		self.elogSearch = KLineEdit(self.searchBar)
		self.btSearch = QPushButton("Find &Next", self.searchBar)
		self.searchLay.addWidget(self.elogSearch)
		self.searchLay.addWidget(self.btSearch)
		self.topBox.addWidget(self.searchBar)
		self.elogSearchComp = self.elogSearch.completionObject()
		self.elogSearch.connect(self.elogSearch, SIGNAL("returnPressed(const QString&)"), self.elogSearchComp.addItem)
		# List view
		self.elogList = KelogList(self.mainBox, self._view_mode)

		# Add HBox for info panel and filters
		self.secondBox = QWidget(self.mainBox)
		self.secondLay = QHBoxLayout(self.secondBox)
		self.elogInfo = KelogInfo(self.secondBox)
		self.elogFilter = KelogFilter(self.secondBox)
		self.secondLay.addWidget(self.elogInfo)
		self.secondLay.addWidget(self.elogFilter)
		self.topBox.addWidget(self.secondBox)

	def initStatusBar (self):
		# Status bar at the bottom
		self.statusBar().showMessage("Status: Ok", 3000)

	def setStatus (self, txt):
		self.statusBar().showMessage(txt, 3000)

	#Slots for Widgets
	def slotElogSelected (self):
		if self.elogList.currentItem():
			if not self.elogList.currentItem().text(3):
				return
			self.elogInfo.parselog(self.elogList.currentItem().text(3), self.elogFilter)

			self.setStatus("Loaded elog %s/%s (%s)" % (self.elogList.currentItem().text(0),
				self.elogList.currentItem().text(1), self.elogList.currentItem().text(2)))

			self.clearAllAction.setEnabled(True)
			self.deleteAction.setEnabled(True)

	def slotElogFilterChanged (self):
		self.slotElogSelected()

	#Slots for Actions
	def slotRefresh (self):
		self.elogInfo.clear()
		self.elogList.refresh()
		if self.elogList.currentItem():
			self.clearAllAction.setEnabled(True)
			self.deleteAction.setEnabled(True)
			self.setStatus("Refreshed.")
		else:
			self.setStatus("Refreshed but no item found.")

	def slotDelete (self):
		if (self.elogList.currentItem()):
			i = self.elogList.currentItem()
			if self.elogList.deleteElog(i ):
				self.setStatus("Deleted : %s" % i.text(3))
				if not self.elogList.currentItem():
					self.elogInfo.clear()
					self.clearAllAction.setEnabled(False)

	def slotClearAll (self):
		rep = KMessageBox.warningYesNo (self, "The whole elog directory is going to be wiped out.\nAre you sure you want to proceed?", "Delete All")
		if rep == KMessageBox.Yes:
			self.elogList.deleteAllElogs()
			self.elogInfo.clear()
			self.setStatus("All Elogs have been cleared.")
			self.clearAllAction.setEnabled(False)


	def slotQuit (self):
		self.close()

	def slotCheckAll (self):
		self.elogFilter.checkAll()
		self.setStatus("All filters activated.")

	def slotToggle (self):
		self.elogFilter.toggle()
		self.setStatus("Filters toggled.")

	def slotExpandTree (self):
		self.elogList.expand()

	def slotCollapseTree (self):
		self.elogList.collapse()

	def slotSwitchViewMode (self):
		self._view_mode = not self._view_mode
		if (self._view_mode == self.VIEW_MODE_TREE):
			self.collapseTreeAction.setEnabled(True)
			self.expandTreeAction.setEnabled(True)
		else:
			self.collapseTreeAction.setEnabled(False)
			self.expandTreeAction.setEnabled(False)
		self.elogList.setTreeViewMode(self._view_mode)
		self.elogList.refresh()


	def slotSearch (self):
		text = self.elogSearch.text()
		if not text:
			self.setStatus("No search string given")
			return
		res = self.elogList.search(text)
		if res == KelogList.NO_MATCH:
			self.setStatus("%s not found" % text)
		elif res == KelogList.NO_OTHER_MATCH:
			self.setStatus("No more matches for %s" % text)


####################################
# Main
####################################

application = ki18n("KElogViewer")
description = ki18n("A KDE elog viewer")
version = "0.5"
copyrt = ki18n("(C) 2006 Wickersheimer Jeremy\n(C) 2010 Michał Ziąbkowski")
aboutData = KAboutData ("klogviewer", "", application, version, description,
		KAboutData.License_GPL, copyrt, ki18n("none"), "http://my.opera.com/mziab/blog", "mziab@o2.pl")

aboutData.addAuthor (ki18n("Michał Ziąbkowski"), ki18n("KDE4 port and clean-up"),
		"mziab@o2.pl")
aboutData.addAuthor (ki18n("Wickersheimer Jeremy"), ki18n("Adapted Elogviewer to KDE"),
		"jwickers@gmail.com")
aboutData.addAuthor (ki18n("Mathias Laurin (synss)"), ki18n("Wrote first Elogviewer in py-gtk"),
		"http://forums.gentoo.org/viewtopic-t-465493-highlight-.html")

aboutData.addCredit (ki18n("Timothy Kilbourn (nmbrthry)"), ki18n("Original idea"), "",
		"http://forums.gentoo.org/viewtopic-t-441228-start-0-postdays-0-postorder-asc-highlight-.html" )

aboutData.setProgramIconName("utilities-log-viewer")

KCmdLineArgs.init (sys.argv, aboutData)

app = KApplication()
win = Kelogviewer()
win.show()
app.exec_()

